#!/bin/zsh

# Copyright 2014, 2016, 2019 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

. "$_EXEC/pdiread.sh"
. "$_EXEC/session_lock.sh"

filter="$(REF f)"
order="$(REF o)"

card="$(POST card)"
cardfile="$_DATA/vcard/$card"
attfile="$_DATA/mappings/attendance"

if ! tempfile=$(CHECK_SLOCK "$cardfile"); then
  SET_COOKIE 0 message="NO VALID FILE LOCK"
  REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
  exit 0
elif [ "$(POST tid)" != "$(transid "$tempfile")" ]; then
  SET_COOKIE 0 message="INVALID TRANSACTION ID"
  REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
  exit 0
fi

vcf_escape(){
  for each in "$@"; do
    printf %s\\n "$each" \
    | sed -E ':X;$!{N;bX}; s;\r\n;\n;g; s;([;,\\]);\\\1;g; s;\n;\\n;g;'
  done \
  | sed -E ':X;$!{N;bX}; s;\n;\;;g'
}

# [ "${_POST[hi_select]}" = "list" ] || _POST[hi_company]="${_POST[hi_other]}"
# [ -n "${_POST[hi_company]}${_POST[hi_number]}${_POST[hi_status]}" ] \
# && _POST[X-HEALTH-INSURANCE]="$(vcf_escape "${_POST[hi_company]}" "${_POST[hi_number]}" "${_POST[hi_status]}")"

vcf="$(pdi_load "$cardfile")"

vcf="$(pdi_update_value "$vcf" N 1 "$(vcf_escape "$(POST 1N)" "$(POST 2N)" "$(POST 3N)" "$(POST 4N)" "$(POST 5N)")")"

for field in $(POST_KEYS |grep -xE '[A-Z][A-Z0-9-]*'); do
  for cnt in $(seq 1 $(POST_COUNT "$field")); do
    case "$field" in
      # (TEL)
      #   printf '%s;TYPE=%s:%s\r\n' "${field}" "${_POST[phonetype${key#TEL}]}" "$(vcf_escape "$(POST "$field" "$cnt")")"
      #   ;;
      TEL)
         vcf="$(pdi_update_attrib "$vcf" TEL $cnt TYPE="$(POST teltype $cnt |grep -Exm1 'HOME|WORK|CELL|FAX')")"
         vcf="$(pdi_update_value "$vcf" "$field" "$cnt" "$(vcf_escape "$(POST "$field" "$cnt")")")"
         ;;
      *)
         vcf="$(pdi_update_value "$vcf" "$field" "$cnt" "$(vcf_escape "$(POST "$field" "$cnt")")")"
        ;;
    esac
done; done
[ "$(POST action)" = addfield ] && vcf="$(vcf_update_field "$vcf" "$(POST newfield)" $(($(pdi_count $(POST newfield)) + 1)) '')"

printf '%s' "$vcf" |grep -vx '' >"$tempfile"

case "$(POST action)" in
  addfield)
    REDIRECT "/cards/?o=${order}&f=${filter}&e=${card}"
    ;;
  update)
    # attendance=()
    # for att in attendance attendance{0..100}; do
    #   [ -n "${_POST[$att]}" ] && attendance+=("${_POST[$att]}")
    # done
    # sed -rn 's:^(.+)'$card'$:\1:p' "$attfile" |while read course; do
    #   touch "$_DATA/ical/$course"
    # done
    # sed -i -r '/^(.+)\t'$card'$/d' "$attfile"
    # for each in $attendance; do
    #   echo "$each\t$card"
    # done >>"$attfile"
    # sed -rn 's:^(.+)'$card'$:\1:p' "$attfile" |while read course; do
    #   touch "$_DATA/ical/$course"
    # done

    cp "$tempfile" "$cardfile"
    RELEASE_SLOCK "$cardfile"
    REDIRECT "/cards/?o=${order}&f=${filter}#${card}"
    ;;
  cancel)
    RELEASE_SLOCK "$cardfile"
    [ -f "$cardfile" ] \
    && REDIRECT "/cards/?o=${order}&f=${filter}#${card}" \
    || REDIRECT "/cards/?o=${order}&f=${filter}"
    ;;
  delete)
    rm "$cardfile"
    RELEASE_SLOCK "$cardfile"
    REDIRECT "/cards/?o=${order}&f=${filter}"
    ;;
esac
